@@ -1,10 +1,12 @@
-import asyncio
 import typing
+from contextlib import asynccontextmanager
+
 from httpx import AsyncClient, Auth, Client, Request, Response, USE_CLIENT_DEFAULT
+from anyio import Lock  # Import after httpx so import errors refer to httpx
 from authlib.common.urls import url_decode
 from authlib.oauth2.client import OAuth2Client as _OAuth2Client
 from authlib.oauth2.auth import ClientAuth, TokenAuth
-from .utils import HTTPX_CLIENT_KWARGS
+from .utils import HTTPX_CLIENT_KWARGS, build_request
 from ..base_client import (
     OAuthError,
     InvalidTokenError,
@@ -14,7 +16,7 @@ from ..base_client import (
 
 __all__ = [
     'OAuth2Auth', 'OAuth2ClientAuth',
-    'AsyncOAuth2Client',
+    'AsyncOAuth2Client', 'OAuth2Client',
 ]
 
 
@@ -27,7 +29,7 @@ class OAuth2Auth(Auth, TokenAuth):
             url, headers, body = self.prepare(
                 str(request.url), request.headers, request.content)
             headers['Content-Length'] = str(len(body))
-            yield Request(method=request.method, url=url, headers=headers, data=body)
+            yield build_request(url=url, headers=headers, body=body, initial_request=request)
         except KeyError as error:
             description = 'Unsupported token_type: {}'.format(str(error))
             raise UnsupportedTokenTypeError(description=description)
@@ -40,7 +42,7 @@ class OAuth2ClientAuth(Auth, ClientAuth):
         url, headers, body = self.prepare(
             request.method, str(request.url), request.headers, request.content)
         headers['Content-Length'] = str(len(body))
-        yield Request(method=request.method, url=url, headers=headers, data=body)
+        yield build_request(url=url, headers=headers, body=body, initial_request=request)
 
 
 class AsyncOAuth2Client(_OAuth2Client, AsyncClient):
@@ -48,6 +50,7 @@ class AsyncOAuth2Client(_OAuth2Client, AsyncClient):
 
     client_auth_class = OAuth2ClientAuth
     token_auth_class = OAuth2Auth
+    oauth_error_class = OAuthError
 
     def __init__(self, client_id=None, client_secret=None,
                  token_endpoint_auth_method=None,
@@ -60,10 +63,9 @@ class AsyncOAuth2Client(_OAuth2Client, AsyncClient):
         client_kwargs = self._extract_session_request_params(kwargs)
         AsyncClient.__init__(self, **client_kwargs)
 
-        # We use a "reverse" Event to synchronize coroutines to prevent
+        # We use a Lock to synchronize coroutines to prevent
         # multiple concurrent attempts to refresh the same token
-        self._token_refresh_event = asyncio.Event()
-        self._token_refresh_event.set()
+        self._token_refresh_lock = Lock()
 
         _OAuth2Client.__init__(
             self, session=None,
@@ -75,42 +77,46 @@ class AsyncOAuth2Client(_OAuth2Client, AsyncClient):
             update_token=update_token, **kwargs
         )
 
-    @staticmethod
-    def handle_error(error_type, error_description):
-        raise OAuthError(error_type, error_description)
-
     async def request(self, method, url, withhold_token=False, auth=USE_CLIENT_DEFAULT, **kwargs):
         if not withhold_token and auth is USE_CLIENT_DEFAULT:
             if not self.token:
                 raise MissingTokenError()
 
-            if self.token.is_expired():
-                await self.ensure_active_token()
+            await self.ensure_active_token(self.token)
 
             auth = self.token_auth
 
         return await super(AsyncOAuth2Client, self).request(
             method, url, auth=auth, **kwargs)
 
-    async def ensure_active_token(self):
-        if self._token_refresh_event.is_set():
-            # Unset the event so other coroutines don't try to update the token
-            self._token_refresh_event.clear()
-            refresh_token = self.token.get('refresh_token')
-            url = self.metadata.get('token_endpoint')
-            if refresh_token and url:
-                await self.refresh_token(url, refresh_token=refresh_token)
-            elif self.metadata.get('grant_type') == 'client_credentials':
-                access_token = self.token['access_token']
-                token = await self.fetch_token(url, grant_type='client_credentials')
-                if self.update_token:
-                    await self.update_token(token, access_token=access_token)
-            else:
-                raise InvalidTokenError()
-            # Notify coroutines that token is refreshed
-            self._token_refresh_event.set()
-            return
-        await self._token_refresh_event.wait()  # wait until the token is ready
+    @asynccontextmanager
+    async def stream(self, method, url, withhold_token=False, auth=USE_CLIENT_DEFAULT, **kwargs):
+        if not withhold_token and auth is USE_CLIENT_DEFAULT:
+            if not self.token:
+                raise MissingTokenError()
+
+            await self.ensure_active_token(self.token)
+
+            auth = self.token_auth
+
+        async with super(AsyncOAuth2Client, self).stream(
+            method, url, auth=auth, **kwargs) as resp:
+            yield resp
+
+    async def ensure_active_token(self, token):
+        async with self._token_refresh_lock:
+            if self.token.is_expired():
+                refresh_token = token.get('refresh_token')
+                url = self.metadata.get('token_endpoint')
+                if refresh_token and url:
+                    await self.refresh_token(url, refresh_token=refresh_token)
+                elif self.metadata.get('grant_type') == 'client_credentials':
+                    access_token = token['access_token']
+                    new_token = await self.fetch_token(url, grant_type='client_credentials')
+                    if self.update_token:
+                        await self.update_token(new_token, access_token=access_token)
+                else:
+                    raise InvalidTokenError()
 
     async def _fetch_token(self, url, body='', headers=None, auth=USE_CLIENT_DEFAULT,
                            method='POST', **kwargs):
@@ -128,7 +134,7 @@ class AsyncOAuth2Client(_OAuth2Client, AsyncClient):
         for hook in self.compliance_hook['access_token_response']:
             resp = hook(resp)
 
-        return self.parse_response_token(resp.json())
+        return self.parse_response_token(resp)
 
     async def _refresh_token(self, url, refresh_token=None, body='',
                              headers=None, auth=USE_CLIENT_DEFAULT, **kwargs):
@@ -139,7 +145,7 @@ class AsyncOAuth2Client(_OAuth2Client, AsyncClient):
         for hook in self.compliance_hook['refresh_token_response']:
             resp = hook(resp)
 
-        token = self.parse_response_token(resp.json())
+        token = self.parse_response_token(resp)
         if 'refresh_token' not in token:
             self.token['refresh_token'] = refresh_token
 
@@ -159,6 +165,7 @@ class OAuth2Client(_OAuth2Client, Client):
 
     client_auth_class = OAuth2ClientAuth
     token_auth_class = OAuth2Auth
+    oauth_error_class = OAuthError
 
     def __init__(self, client_id=None, client_secret=None,
                  token_endpoint_auth_method=None,
@@ -190,23 +197,23 @@ class OAuth2Client(_OAuth2Client, Client):
             if not self.token:
                 raise MissingTokenError()
 
-            if self.token.is_expired():
-                self.ensure_active_token()
+            if not self.ensure_active_token(self.token):
+                raise InvalidTokenError()
 
             auth = self.token_auth
 
         return super(OAuth2Client, self).request(
             method, url, auth=auth, **kwargs)
 
-    def ensure_active_token(self):
-        refresh_token = self.token.get('refresh_token')
-        url = self.metadata.get('token_endpoint')
-        if refresh_token and url:
-            self.refresh_token(url, refresh_token=refresh_token)
-        elif self.metadata.get('grant_type') == 'client_credentials':
-            access_token = self.token['access_token']
-            token = self.fetch_token(url, grant_type='client_credentials')
-            if self.update_token:
-                self.update_token(token, access_token=access_token)
-        else:
-            raise InvalidTokenError()
+    def stream(self, method, url, withhold_token=False, auth=USE_CLIENT_DEFAULT, **kwargs):
+        if not withhold_token and auth is USE_CLIENT_DEFAULT:
+            if not self.token:
+                raise MissingTokenError()
+
+            if not self.ensure_active_token(self.token):
+                raise InvalidTokenError()
+
+            auth = self.token_auth
+
+        return super(OAuth2Client, self).stream(
+            method, url, auth=auth, **kwargs)
