@@ -1,5 +1,4 @@
 from authlib.common.urls import add_params_to_qs
-from authlib.deprecate import deprecate
 from .assertion import client_secret_jwt_sign, private_key_jwt_sign
 from .client import ASSERTION_TYPE
 
@@ -23,14 +22,18 @@ class ClientSecretJWT(object):
 
     :param token_endpoint: A string URL of the token endpoint
     :param claims: Extra JWT claims
+    :param headers: Extra JWT headers
+    :param alg: ``alg`` value, default is HS256
     """
     name = 'client_secret_jwt'
     alg = 'HS256'
 
-    def __init__(self, token_endpoint=None, claims=None, header=None):
+    def __init__(self, token_endpoint=None, claims=None, headers=None, alg=None):
         self.token_endpoint = token_endpoint
         self.claims = claims
-        self.header = header
+        self.headers = headers
+        if alg is not None:
+            self.alg = alg
 
     def sign(self, auth, token_endpoint):
         return client_secret_jwt_sign(
@@ -38,7 +41,7 @@ class ClientSecretJWT(object):
             client_id=auth.client_id,
             token_endpoint=token_endpoint,
             claims=self.claims,
-            header=self.header,
+            headers=self.headers,
             alg=self.alg,
         )
 
@@ -74,6 +77,8 @@ class PrivateKeyJWT(ClientSecretJWT):
 
     :param token_endpoint: A string URL of the token endpoint
     :param claims: Extra JWT claims
+    :param headers: Extra JWT headers
+    :param alg: ``alg`` value, default is RS256
     """
     name = 'private_key_jwt'
     alg = 'RS256'
@@ -84,24 +89,5 @@ class PrivateKeyJWT(ClientSecretJWT):
             client_id=auth.client_id,
             token_endpoint=token_endpoint,
             claims=self.claims,
-            header=self.header,
             alg=self.alg,
         )
-
-
-def register_session_client_auth_method(session, token_url=None, **kwargs):  # pragma: no cover
-    """Register "client_secret_jwt" or "private_key_jwt" token endpoint auth
-    method to OAuth2Session.
-
-    :param session: OAuth2Session instance.
-    :param token_url: Optional token endpoint url.
-    """
-    deprecate('Use `ClientSecretJWT` and `PrivateKeyJWT` instead', '1.0', 'Jeclj', 'ca')
-    if session.token_endpoint_auth_method == 'client_secret_jwt':
-        cls = ClientSecretJWT
-    elif session.token_endpoint_auth_method == 'private_key_jwt':
-        cls = PrivateKeyJWT
-    else:
-        raise ValueError('Invalid token_endpoint_auth_method')
-
-    session.register_client_auth_method(cls(token_url))
