@@ -27,13 +27,4 @@ INSTALLED_APPS=[
     'tests.django.test_oauth2',
 ]
 
-AUTHLIB_OAUTH_CLIENTS = {
-    'dev_overwrite': {
-        'client_id': 'dev-client-id',
-        'client_secret': 'dev-client-secret',
-        'access_token_params': {
-            'foo': 'foo-1',
-            'bar': 'bar-2'
-        }
-    }
-}
+USE_TZ = True
